
#skript pro vypocet matice poctu a jeji normalizaci


#nastaveni cesty k soubory s daty
path <- "" #sem vlozte cestu k datum
setwd("path")

#vytvoreni vektoru nazvu BAM souboru
B <- paste("B",  sprintf("%s_sorted.bam", 1:6), sep="")
C <- paste("C",  sprintf("%s_sorted.bam", 1:6), sep="")
D <- paste("D",  sprintf("%s_sorted.bam", 1:6), sep="")
E <- paste("E",  sprintf("%s_sorted.bam", 1:6), sep="")

#funkce z knihovny Rsubread k vytvoreni count tables, kdy byly geny anotovane podle .gff3 souboru C. beijerinckii NRRL B-598 z NCBI 
library(Rsubread) 
countTable1 <- featureCounts(files = c(B,C,D,E), annot.ext="sequence.gff3", isGTFAnnotationFile=TRUE, GTF.featureType="gene",  GTF.attrType="locus_tag", strandSpecific=2)
countTable2 <- featureCounts(files = c(B,C,D,E), annot.ext="sequence.gff3", isGTFAnnotationFile=TRUE, GTF.featureType="pseudogene",  GTF.attrType="locus_tag", strandSpecific=2)

library(DESeq2)

#ziskani poctu prirazenych cteni ke vzorku v urcity cas mereni
coldata1 <- countTable1$stat
rownames(coldata1) <- (coldata1[, 1])
coldata1 <- coldata1[,-1]
coldata1 <- t(coldata1)

Assigned1 <- factor(coldata1[,1])

coldata2 <- countTable2$stat
rownames(coldata2) <- (coldata2[, 1])
coldata2 <- coldata2[,-1]
coldata2 <- t(coldata2)

Assigned2 <- factor(coldata2[,1])

#ziskani count table
cts1 <- countTable1$counts
cts2 <- countTable2$counts

#vypocitani normalizovane count table pomoci DESeq2 metodou pomeru medianu
dds1 <- DESeqDataSetFromMatrix(countData = cts1, colData = DataFrame(Assigned1), design = ~ Assigned1)
dds1 <- estimateSizeFactors(dds1)
normalized_counts1 <- counts(dds1, normalized=TRUE)

dds2 <- DESeqDataSetFromMatrix(countData = cts2, colData = DataFrame(Assigned2), design = ~ Assigned2)
dds2 <- estimateSizeFactors(dds2)
normalized_counts2 <- counts(dds2, normalized=TRUE)

normalized_counts <- rbind(normalized_counts1, normalized_counts2)

#vykresleni exprese genu vzorku podle casu pomoci heatmap
B_heatmap <- heatmap(normalized_counts[,1:6], Colv = NA, Rowv = NA, labCol = (c("T1","T2","T3","T4","T5","T6")), main = "Vzorek B")
C_heatmap <- heatmap(normalized_counts[,7:12], Colv = NA, Rowv = NA, labCol = (c("T1","T2","T3","T4","T5","T6")), main = "Vzorek C")
D_heatmap <- heatmap(normalized_counts[,13:18], Colv = NA, Rowv = NA, labCol = (c("T1","T2","T3","T4","T5","T6")), main = "Vzorek D")
E_heatmap <- heatmap(normalized_counts[,19:24], Colv = NA, Rowv = NA, labCol = (c("T1","T2","T3","T4","T5","T6")), main = "Vzorek E")

#ulozeni normalizovane count table
write.table(normalized_counts,file="normalized_counts.csv",quote=FALSE,sep=";")
