###################################################
###skript pro nacteni normalizovane count_table.csv

path <- ""    #nastaveni vlastni cesty ke vstupnim datum count table
setwd("path")
count_table <- read.csv("normalized_counts.csv", sep = ";")

#count table podle casu
count_table_1 <- count_table[,seq(1, 19, by = 6)]
count_table_2 <- count_table[,seq(2, 20, by = 6)]
count_table_3 <- count_table[,seq(3, 21, by = 6)]
count_table_4 <- count_table[,seq(4, 22, by = 6)]
count_table_5 <- count_table[,seq(5, 23, by = 6)]
count_table_6 <- count_table[,seq(6, 24, by = 6)]

time_name <- c("count_table_1","count_table_2","count_table_3","count_table_4","count_table_5","count_table_6")

#vytvoreni matice s expresi genu na konkretnich pozicich
count_table_Time <- matrix(list(), dim(count_table)[1], dim(count_table)[2]/4)
for (i in 1:dim(count_table_Time)[2]){
  time <- get(time_name[i])
  for (j in 1:dim(count_table_Time)[1]){
    one_time <- time[j,]
    count_table_Time[[j,i]] <- one_time
  }
}

names <- row.names(count_table)


#############################################################################################
#############################################################################################
#############################################################################################

vypocet_GRN <- function(count_table_Time, name){
  #funkce pro vypocet adjacency matrix
  #vstupy:
  #count_table_Time - obsahuje data genove exprese z nekolika casovych bodu z vice experimentu
  #                 - jedna se o list, ktery na radcich ma geny a ve sloupcich casove body
  #                 - na jedne pozicic je ulozeno vice hodnot genove exprese
  #name - obsahuje nazvy genu v poradi jako u count_table_Time
  
  
#############################################################################################
##casovac
library(pracma)
tic(gcFirst=FALSE)

#############################################################################################
## pocatecni zmena exprese
# pozice (casova), kdy doslo k vyznamne zmene genove exprese
# NaN pokud zmena neni vyznamna

num_exp <- length(count_table_Time[[1,1]])

t_down <- 0.6
t_up <- 1/t_down

start_posit <- matrix(,dim(count_table_Time)[1],num_exp)


for (y in 1:num_exp) {
  for (i in 1:dim(count_table_Time)[1]) {
    count <- 2
    
    g_0 <- count_table_Time[[i, 1]][y]
    g_x <- count_table_Time[[i, count]][y]
    
    if (g_0 == 0) {
      #osetreni deleni nulou
      multipl <- 1
    } else{
      multipl <- g_x / g_0
    }
    
    while (dim(count_table_Time)[2] > count && multipl < t_up && multipl > t_down) {
      count <- count + 1
      g_x <- count_table_Time[[i, count]][y]
      
      
      if (g_0 == 0 && g_x == 0) {
        #osetreni deleni nulou
        multipl <- 1
      } else if (g_0 == 0 && g_x != g_0) {
        multipl <- 2
      } else{
        multipl <- g_x / g_0
      }
      
    }
    
    if (count == dim(count_table_Time)[2]) {
      if (multipl >= t_up || multipl <= t_down) {
        start_posit[i, y] <- count
      } else{
        start_posit[i, y] <- dim(count_table_Time)[2] + 1
      }
      
    } else{
      start_posit[i, y] <- count
    }
  }
}


absolute_start <- apply(start_posit,1,FUN=min) #nejmensi zacinajici pozice pro gen ze vsech experimentu

#############################################################################################
## vypocet vzajemne informace
library(GenKern)

time_length <- dim(count_table_Time)[2]
MI_list <- matrix(list(), dim(count_table_Time)[1], dim(count_table_Time)[2])

#nastaveni k fci KernSec
xgs <- 50
ygs <- 50

for(list_count_i in 1:dim(MI_list)[1]){
  
  for(list_count_j in 1:dim(MI_list)[2]){
    
    MI_list[[list_count_i,list_count_j]] <- matrix(0,dim(count_table_Time)[1],dim(count_table_Time)[2])
    
    if(list_count_j >= absolute_start[list_count_i]){
      
      init_expresion <- unlist(count_table_Time[list_count_i,list_count_j])
      if(std(init_expresion) == 0){
        init_expresion[1] <- init_expresion[1] + 1e-100
      }
      
      bandx <- tryCatch(dpik(init_expresion, scalest = "minim", level = 10L, kernel = "normal"), error = function(err) 0.2)
      Px <- KernSec(init_expresion, xgridsize = xgs, xbandwidth = bandx)$yden
      Px <- Px/sum(Px)
      
      for(i in 1:dim(count_table_Time)[1]){
        
        if(list_count_i != i){
          row_start <- absolute_start[i]
          
          for(y in 1:dim(count_table_Time)[2]) {
            
            if(row_start <= y  && list_count_j <= y){
              
              actual_expresion <- unlist(count_table_Time[i,y])
              if(std(actual_expresion) == 0){
                actual_expresion[1] <- actual_expresion[1] + 1e-100
              }
              
              bandy <- tryCatch(dpik(actual_expresion, scalest = "minim", level = 10L, kernel = "normal"), error = function(err) 0.2)
              Py <- KernSec(actual_expresion, xgridsize = ygs, xbandwidth = bandy)$yden
              Py <- Py/sum(Py)
              Pxy <- KernSur(init_expresion, actual_expresion, xgridsize = xgs, ygridsize = ygs, xbandwidth = bandx, ybandwidth = bandy)$zden
              Pxy <- Pxy/sum(Pxy)
              
              MI <- 0
              for (MI_i in 1:xgs) for (MI_j in 1:ygs) {
                tmp <- Pxy[MI_i, MI_j] * log2(Pxy[MI_i, MI_j] / Px[MI_i] / Py[MI_j])
                if (tmp != "NaN") 
                  MI <- MI + tmp
              }
              
              MI_list[[list_count_i, list_count_j]][i,y] <- MI
              
            }
          }
        }
      }
    }
  }
}

#############################################################################################
##boostrap
library(tseries)

n <- 3 #pocet provedeni boostrapu
MI_boot <- c()

for(num_of_bootstraping in 1:n ){
  index <- t(tsbootstrap(1:time_length, nb = dim(count_table_Time)[1], type = c("stationary","block")))
  bootstraped <- count_table_Time
  
  for(i in 1:dim(count_table_Time)[1]){
    bootstraped[i,] <- count_table_Time[i,index[i,]]
  }
  
  for(num_boot1 in 1:dim(bootstraped)[1]){
    for(num_boot2 in 1:dim(bootstraped)[2]){
      
      init_expresion <- unlist(bootstraped[num_boot1,num_boot2])
      if(std(init_expresion) == 0){
        init_expresion[1] <- init_expresion[1] + 1e-100
      }
      
      bandx <- tryCatch(dpik(init_expresion, scalest = "minim", level = 10L, kernel = "normal"), error = function(err) 0.2)
      Px <- KernSec(init_expresion, xgridsize = xgs, xbandwidth = bandx)$yden
      Px <- Px/sum(Px)
      
      for(z in 1:dim(bootstraped)[1]){
        for(y in 1:dim(bootstraped)[2]) {
          
          if (num_boot2 <= y && num_boot1 != z) {
            actual_expresion <- unlist(bootstraped[z,y])
            if(std(actual_expresion) == 0){
              actual_expresion[1] <- actual_expresion[1] + 1e-100
            }
            
            bandy <- tryCatch(dpik(actual_expresion, scalest = "minim", level = 10L, kernel = "normal"), error = function(err) 0.2)
            Py <- KernSec(actual_expresion, xgridsize = ygs, xbandwidth = bandy)$yden
            Py <- Py/sum(Py)
            Pxy <- KernSur(init_expresion, actual_expresion, xgridsize = xgs, ygridsize = ygs, xbandwidth = bandx, ybandwidth = bandy)$zden
            Pxy <- Pxy/sum(Pxy)
            
            MI <- 0
            for (i in 1:xgs) for (j in 1:ygs) {
              tmp <- Pxy[i, j] * log2(Pxy[i, j] / Px[i] / Py[j])
              if (tmp != "NaN" && tmp != Inf) 
                MI <- MI + tmp
            }
            
            MI_boot[length(MI_boot)+1] <- MI
            
          } 
        }
      }
    }
  }
}

hole_mean <- mean(MI_boot)
hole_std <- std(MI_boot) 

#############################################################################################
##filtrace

alpha <- 0.05
treshold <- hole_mean + alpha * hole_std

filt_inter <- matrix(list(), dim(MI_list)[1], dim(MI_list)[2])

for(i in 1:dim(MI_list)[1]){
  for(y in 1:dim(MI_list)[2]){
    
    H_MI_table <- MI_list[[i,y]]
    H_MI_table[H_MI_table < treshold] <- 0
    H_MI_table[H_MI_table > treshold] <- 1
    
    filt_inter[[i,y]] <- H_MI_table
  }
}

#############################################################################################
##matice reakci

gen_inter <- matrix(0,dim(MI_list)[1],dim(MI_list)[1])
gen_inter_MI <- matrix(0,dim(MI_list)[1],dim(MI_list)[1])
start_end <- matrix(list(), dim(MI_list)[1], dim(MI_list)[1])

for(gen1 in 1:dim(MI_list)[1]){
  for(gen2 in 1:dim(MI_list)[1]){
    for(MI_list_length in 1:dim(MI_list)[2]){
      position <- c()
      H_Sig_inter <- filt_inter[[gen1,MI_list_length]]
      concrete_row <- H_Sig_inter[gen2,]
      position <- which(concrete_row == 1)
      if(length(position) != 0){
        gen_inter[gen1,gen2] <- 1
        H_MI_table <- MI_list[[gen1,MI_list_length]]
        gen_inter_MI[gen1,gen2] <- H_MI_table[gen2,position[1]]
        start_end[[gen1,gen2]] <- c(MI_list_length,position[1])
        break
      }
    }
  }
}

#############################################################################################
##DPI
gen_inter_DPI <- gen_inter

for(gen1 in 1:dim(MI_list)[1]){
  for(gen2 in 1:dim(MI_list)[1]){
    for(gen3 in 1:dim(MI_list)[1]){
      if((gen1 != gen2 && gen1 != gen3 && gen2 != gen3) && gen_inter[gen1,gen2] == 1 && gen_inter[gen1,gen3] == 1 && gen_inter[gen2,gen3] == 1){
        if(gen_inter_MI[gen1,gen2]<=min(c(gen_inter_MI[gen1,gen3], gen_inter_MI[gen3,gen2]))){
          gen_inter_DPI[gen1,gen2] <- 0
        }        
      }
    }
  }
}


#############################################################################################
##vysledna matice

adjacency_matrix <- matrix(0,dim(MI_list)[1],dim(MI_list)[1])

for(gen1 in 1:dim(MI_list)[1]){
  for(gen2 in 1:dim(MI_list)[1]){
    if(gen_inter_DPI[gen1,gen2] == 1){
      G1 <- start_end[[gen1,gen2]][1]
      G2 <- start_end[[gen1,gen2]][2]
      
      gen1_pre <- mean(unlist(count_table_Time[[gen1,G1-1]]))
      gen1_a <- mean(unlist(count_table_Time[[gen1,G1]]))
      gen2_pre <- mean(unlist(count_table_Time[[gen2,G2-1]]))
      gen2_a <- mean(unlist(count_table_Time[[gen2,G2]]))      
      
      if((gen1_pre < gen1_a && gen2_pre < gen2_a) || (gen1_pre > gen1_a && gen2_pre > gen2_a)){
        adjacency_matrix[gen1,gen2] <- 1
      } else if ((gen1_pre < gen1_a && gen2_pre > gen2_a) || (gen1_pre > gen1_a && gen2_pre < gen2_a)){
        adjacency_matrix[gen1,gen2] <- -1
      }
    }
  }
}

#############################################################################################
##casovac
toc(echo=TRUE)

rownames(adjacency_matrix) <- names
colnames(adjacency_matrix) <- names

#ulozeni adjacency matrix
write.table(adjacency_matrix,file="adjacency_matrix",quote=FALSE,sep=";")
}
